searchData={"items":[{"type":"module","title":"Membrane.H264.Parser","doc":"Membrane element providing parser for H264 encoded video stream.\n\nThe parser:\n* prepares and sends the appropriate stream format, based on information provided in the stream and via the element's options\n* splits the incoming stream into H264 access units - each buffer being output is a `Membrane.Buffer` struct with a\nbinary payload of a single access unit or network abstraction layer unit.\n* enriches the output buffers with the metadata describing the way the access unit is split into NAL units, type of each NAL unit\nmaking up the access unit and the information if the access unit hold a keyframe.\n* converts the stream's structure (Annex B, avc1 or avc3) to the one provided via the element's options.\n\nThe parser works in one of three possible modes, depending on the structure of the input buffers:\n* `:bytestream` - each input buffer contains some part of H264 stream's payload, but not necessary a logical\nH264 unit (like NAL unit or an access unit). Can be used for i.e. for parsing the stream read from the file.\n* `:nalu_aligned` - each input buffer contains a single NAL unit's payload\n* `:au_aligned` - each input buffer contains a single access unit's payload\n\nThe parser's mode is set automatically, based on the input stream format received by that element:\n* Receiving `%Membrane.RemoteStream{type: :bytestream}` results in the parser mode being set to `:bytestream`\n* Receiving `%Membrane.H264{alignment: :nalu}` results in the parser mode being set to `:nalu_aligned`\n* Receiving `%Membrane.H264{alignment: :au}` results in the parser mode being set to `:au_aligned`\n\nThe distinction between parser modes was introduced to eliminate the redundant operations and to provide a reliable way\nfor rewriting of timestamps:\n* in the `:bytestream` mode:\n  * if option `:framerate` is set to nil, the output buffers have their `:pts` and `:dts` set to nil\n  * if framerate is specified, `:pts` and `:dts` will be generated automatically, based on that framerate, starting from 0\n   This may only be used with H264 profiles `:baseline` and `:constrained_baseline`, where `PTS==DTS`.\n* in the `:nalu_aligned` mode, the output buffers have their `:pts` and `:dts` set to `:pts` and `:dts` of the\n input buffer that was holding the first NAL unit making up given access unit (that is being sent inside that output buffer).\n* in the `:au_aligned` mode, the output buffers have their `:pts` and `:dts` set to `:pts` and `:dts` of the input buffer\n(holding the whole access unit being output)\n\nThe parser also allows for conversion between stream structures. The available structures are:\n* Annex B, `:annexb` - In a stream with this structure each NAL unit is prefixed by three or\nfour-byte start code (`0x(00)000001`) that allows to identify boundaries between them.\n* avc1, `:avc1` - In such stream a DCR (Decoder Configuration Record) is included in `stream_format`\nand NALUs lack the start codes, but are prefixed with their length. The length of these prefixes\nis contained in the stream's DCR. PPSs and SPSs (Picture Parameter Sets and Sequence Parameter Sets) are\ntransported in the DCR.\n* avc3, `:avc3` - The same as avc1, only that parameter sets may be also present in the stream\n(in-band).","ref":"Membrane.H264.Parser.html"},{"type":"module","title":"Element options - Membrane.H264.Parser","doc":"Passed via struct `t:Membrane.H264.Parser.t/0`\n\n- `spss`  \n\n  ```\n  [binary()]\n  ```\n  \n  Default value: `[]`  \n  Sequence Parameter Set NAL unit binary payloads - if absent in the stream, should\n  be provided via this option (only available for `:annexb` output stream structure).\n\n\n- `ppss`  \n\n  ```\n  [binary()]\n  ```\n  \n  Default value: `[]`  \n  Picture Parameter Set NAL unit binary payloads - if absent in the stream, should\n  be provided via this option (only available for `:annexb` output stream structure).\n\n\n\n- `output_alignment`  \n\n  ```\n  :au | :nalu\n  ```\n  \n  Default value: `:au`  \n  Alignment of the buffers produced as an output of the parser.\n  If set to `:au`, each output buffer will be a single access unit.\n  Otherwise, if set to `:nalu`, each output buffer will be a single NAL unit.\n  Defaults to `:au`.\n\n\n\n- `skip_until_keyframe`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `true`  \n  Determines whether to drop the stream until the first key frame is received.\n\n\n\n- `repeat_parameter_sets`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `false`  \n  Repeat all parameter sets (`sps` and `pps`) on each IDR picture.\n  \n  Parameter sets may be retrieved from:\n    * The stream\n    * `Parser` options.\n    * Decoder Configuration Record, sent in `:acv1` and `:avc3` stream types\n\n\n\n- `output_stream_structure`  \n\n  ```\n  nil | :annexb | :avc1 | :avc3 | {:avc1 | :avc3, nalu_length_size :: pos_integer()}\n  ```\n  \n  Default value: `nil`  \n  format of the outgoing H264 stream, if set to `:annexb` NALUs will be separated by\n  a start code (0x(00)000001) or if set to `:avc3` or `:avc1` they will be prefixed by their size.\n  Additionally for `:avc1` and `:avc3` a tuple can be passed containing the atom and\n  `nalu_length_size` that determines the size in bytes of each NALU's field\n  describing their length (by default 4). In avc1 output streams the PPSs and SPSs will be\n  transported in the DCR, when in avc3 they will be present only in the stream (in-band).\n  If not provided or set to nil the stream's structure will remain unchanged.\n\n\n\n- `generate_best_effort_timestamps`  \n\n  ```\n  false | %{:framerate => {pos_integer(), pos_integer()}, optional(:add_dts_offset) => boolean()}\n  ```\n  \n  Default value: `false`  \n  Generates timestamps based on given `framerate`.\n  \n  This option works only when `Membrane.RemoteStream` format arrives.\n  \n  Keep in mind that the generated timestamps may be inaccurate and lead\n  to video getting out of sync with other media, therefore h264 should\n  be kept in a container that stores the timestamps alongside.\n  \n  By default, the parser adds negative DTS offset to the timestamps,\n  so that in case of frame reorder (which always happens when B frames\n  are present) the DTS was always bigger than PTS. If that is not desired,\n  you can set `add_dts_offset: false`.","ref":"Membrane.H264.Parser.html#module-element-options"},{"type":"module","title":"Pads - Membrane.H264.Parser","doc":"### `:input`\n\nAccepted formats:\n```\n%RemoteStream{type: :bytestream}\n```\n```\nH264\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:auto`\nDemand unit: | `:buffers`\n\n\n### `:output`\n\nAccepted formats:\n```\n%H264{alignment: alignment, nalu_in_metadata?: true} when alignment in [:nalu, :au]\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:auto`\nDemand unit: | `nil`","ref":"Membrane.H264.Parser.html#module-pads"},{"type":"function","title":"Membrane.H264.Parser.options/0","doc":"Returns description of options available for this module","ref":"Membrane.H264.Parser.html#options/0"},{"type":"type","title":"Membrane.H264.Parser.stream_structure/0","doc":"Type referencing `Membrane.H264.stream_structure` type, in case of `:avc1` and `:avc3`\nstream structure, it contains an information about the size of each NALU's prefix describing\ntheir length.","ref":"Membrane.H264.Parser.html#t:stream_structure/0"},{"type":"type","title":"Membrane.H264.Parser.t/0","doc":"Struct containing options for `Membrane.H264.Parser`","ref":"Membrane.H264.Parser.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.AUSplitter","doc":"Module providing functionalities to divide the binary\nh264 stream into access units.\n\nThe access unit splitter's behaviour is based on *\"7.4.1.2.3\nOrder of NAL units and coded pictures and association to access units\"*\nof the *\"ITU-T Rec. H.264 (01/2012)\"* specification. The most crucial part\nof the access unit splitter is the mechanism to detect new primary coded video picture.\n\nWARNING: Our implementation of that mechanism is based on:\n*\"7.4.1.2.4 Detection of the first VCL NAL unit of a primary coded picture\"*\nof the *\"ITU-T Rec. H.264 (01/2012)\"*, however it adds one more\nadditional condition which, when satisfied, says that the given\nVCL NALu is a new primary coded picture. That condition is whether the picture\nis a keyframe or not.","ref":"Membrane.H264.Parser.AUSplitter.html"},{"type":"function","title":"Membrane.H264.Parser.AUSplitter.new/0","doc":"Returns a structure holding a clear state of the\naccess unit splitter.","ref":"Membrane.H264.Parser.AUSplitter.html#new/0"},{"type":"function","title":"Membrane.H264.Parser.AUSplitter.split/3","doc":"Splits the given list of NAL units into the access units.\n\nIt can be used for a stream which is not completely available at the time of function invocation,\nas the function updates the state of the access unit splitter - the function can\nbe invoked once more, with new NAL units and the updated state.\nUnder the hood, `split/2` defines a finite state machine\nwith two states: `:first` and `:second`. The state `:first` describes the state before\nreaching the primary coded picture NALu of a given access unit. The state `:second`\ndescribes the state after processing the primary coded picture NALu of a given\naccess unit.\n\nIf `assume_au_aligned` flag is set to `true`, input is assumed to form a complete set\nof access units and therefore all of them are returned. Otherwise, the last access unit\nis not returned until another access unit starts, as it's the only way to prove that\nthe access unit is complete.","ref":"Membrane.H264.Parser.AUSplitter.html#split/3"},{"type":"type","title":"Membrane.H264.Parser.AUSplitter.access_unit/0","doc":"A type representing an access unit - a list of logically associated NAL units.","ref":"Membrane.H264.Parser.AUSplitter.html#t:access_unit/0"},{"type":"opaque","title":"Membrane.H264.Parser.AUSplitter.t/0","doc":"A structure holding a state of the access unit splitter.","ref":"Membrane.H264.Parser.AUSplitter.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.DecoderConfigurationRecord","doc":"Utility functions for parsing and generating AVC Configuration Record.\n\nThe structure of the record is described in section 5.2.4.1.1 of MPEG-4 part 15 (ISO/IEC 14496-15).","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html"},{"type":"function","title":"Membrane.H264.Parser.DecoderConfigurationRecord.generate/3","doc":"Generates a DCR based on given PPSs and SPSs.","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html#generate/3"},{"type":"function","title":"Membrane.H264.Parser.DecoderConfigurationRecord.parse/1","doc":"Parses the DCR.","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html#parse/1"},{"type":"function","title":"Membrane.H264.Parser.DecoderConfigurationRecord.remove_parameter_sets/1","doc":"","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html#remove_parameter_sets/1"},{"type":"type","title":"Membrane.H264.Parser.DecoderConfigurationRecord.t/0","doc":"Structure representing the Decoder Configuartion Record","ref":"Membrane.H264.Parser.DecoderConfigurationRecord.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.Format","doc":"Module providing functionalities for preparing H264\nformat based on the parsed SPS NAL units.","ref":"Membrane.H264.Parser.Format.html"},{"type":"function","title":"Membrane.H264.Parser.Format.from_sps/3","doc":"Prepares the `Membrane.H264.t()` format based on the parsed SPS NALu.\nDuring the process, the function determines the profile of\nthe h264 stream and the picture resolution.","ref":"Membrane.H264.Parser.Format.html#from_sps/3"},{"type":"module","title":"Membrane.H264.Parser.NALu","doc":"A module defining a struct representing a single NAL unit.","ref":"Membrane.H264.Parser.NALu.html"},{"type":"type","title":"Membrane.H264.Parser.NALu.t/0","doc":"A type defining the structure of a single NAL unit produced\nby the `Membrane.H264.Parser.NALuParser`.\n\nIn the structure there ardqde following fields:\n* `parsed_fields` - the map with keys being the NALu field names and the values being the value fetched from the NALu binary.\nThey correspond to the NALu schemes defined in the section *7.3.* of the *\"ITU-T Rec. H.264 (01/2012)\"*.\n* `stripped_prefix` - prefix that used to split the NAL units in the bytestream and was stripped from the payload.\nThe prefix is defined as in: *\"Annex B\"* of the *\"ISO/IEC 14496-10\"* or in \"ISO/IEC 14496-15\".\n* `type` - an atom representing the type of the NALu. Atom's name is based on the\n*\"Table 7-1 – NAL unit type codes, syntax element categories, and NAL unit type classes\"* of the *\"ITU-T Rec. H.264 (01/2012)\"*.\n* `payload` - the binary, which parsing resulted in that structure being produced stripped of it's prefix\n* `status` - `:valid`, if the parsing was successfull, `:error` otherwise","ref":"Membrane.H264.Parser.NALu.html#t:t/0"},{"type":"type","title":"Membrane.H264.Parser.NALu.timestamps/0","doc":"","ref":"Membrane.H264.Parser.NALu.html#t:timestamps/0"},{"type":"module","title":"Membrane.H264.Parser.NALuParser","doc":"A module providing functionality of parsing a stream of binaries, out of which each\nis a payload of a single NAL unit.","ref":"Membrane.H264.Parser.NALuParser.html"},{"type":"function","title":"Membrane.H264.Parser.NALuParser.get_prefixed_nalu_payload/3","doc":"Returns payload of the NALu with appropriate prefix generated based on output stream\nstructure and prefix length.","ref":"Membrane.H264.Parser.NALuParser.html#get_prefixed_nalu_payload/3"},{"type":"function","title":"Membrane.H264.Parser.NALuParser.new/1","doc":"Returns a structure holding a clear NALu parser state. `input_stream_structure`\ndetermines the prefixes of input NALU payloads.","ref":"Membrane.H264.Parser.NALuParser.html#new/1"},{"type":"function","title":"Membrane.H264.Parser.NALuParser.parse/4","doc":"Parses a binary representing a single NALu and removes it's prefix (if it exists).\n\nReturns a structure that\ncontains parsed fields fetched from that NALu.\nWhen `payload_prefixed?` is true the input binary is expected to contain one of:\n* prefix defined as the *\"Annex B\"* of the *\"ITU-T Rec. H.264 (01/2012)\"*.\n* prefix of size defined in state describing the length of the NALU in bytes, as described in *ISO/IEC 14496-15*.","ref":"Membrane.H264.Parser.NALuParser.html#parse/4"},{"type":"function","title":"Membrane.H264.Parser.NALuParser.parse_nalus/4","doc":"Parses a list of binaries, each representing a single NALu.\n\nSee `parse/3` for details.","ref":"Membrane.H264.Parser.NALuParser.html#parse_nalus/4"},{"type":"function","title":"Membrane.H264.Parser.NALuParser.prefix_nalus_payloads/2","doc":"","ref":"Membrane.H264.Parser.NALuParser.html#prefix_nalus_payloads/2"},{"type":"opaque","title":"Membrane.H264.Parser.NALuParser.t/0","doc":"A structure holding the state of the NALu parser.","ref":"Membrane.H264.Parser.NALuParser.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.NALuSplitter","doc":"A module with functions responsible for splitting\nthe h264 stream into the NAL units.\n\nThe splitting is based on\n*\"Annex B\"* of the *\"ITU-T Rec. H.264 (01/2012)\"*.","ref":"Membrane.H264.Parser.NALuSplitter.html"},{"type":"function","title":"Membrane.H264.Parser.NALuSplitter.new/2","doc":"Returns a structure holding a NALu splitter state.\n\n\nThe `input_stream_structure` determines which prefix is considered as delimiting two NALUs.\nBy default, the inner `unparsed_payload` of the state is clean, but can be set to a given binary.","ref":"Membrane.H264.Parser.NALuSplitter.html#new/2"},{"type":"function","title":"Membrane.H264.Parser.NALuSplitter.split/3","doc":"Splits the binary into NALus sequence.\n\nTakes a binary h264 stream as an input\nand produces a list of binaries, where each binary is\na complete NALu that can be passed to the `Membrane.H264.Parser.NALuParser.parse/2`.\n\nIf `assume_nalu_aligned` flag is set to `true`, input is assumed to form a complete set\nof NAL units and therefore all of them are returned. Otherwise, the NALu is not returned\nuntil another NALu starts, as it's the only way to prove that the NALu is complete.","ref":"Membrane.H264.Parser.NALuSplitter.html#split/3"},{"type":"opaque","title":"Membrane.H264.Parser.NALuSplitter.t/0","doc":"A structure holding the state of the NALu splitter.","ref":"Membrane.H264.Parser.NALuSplitter.html#t:t/0"},{"type":"module","title":"Membrane.H264.Parser.NALuTypes","doc":"The module aggregating the mapping of from `nal_unit_type`\nfields of the NAL unit to the human-friendly name of a NALu type.","ref":"Membrane.H264.Parser.NALuTypes.html"},{"type":"function","title":"Membrane.H264.Parser.NALuTypes.get_type/1","doc":"The function which returns the human friendly name of a NALu type\nfor a given `nal_unit_type`.\n\nThe mapping is based on: \"Table 7-1 – NAL unit type codes, syntax element categories, and NAL unit type classes\"\nof the *\"ITU-T Rec. H.264 (01/2012)\"*","ref":"Membrane.H264.Parser.NALuTypes.html#get_type/1"},{"type":"macro","title":"Membrane.H264.Parser.NALuTypes.is_vcl_nalu_type/1","doc":"","ref":"Membrane.H264.Parser.NALuTypes.html#is_vcl_nalu_type/1"},{"type":"type","title":"Membrane.H264.Parser.NALuTypes.nalu_type/0","doc":"A type representing all the possible human-friendly names of NAL unit types.","ref":"Membrane.H264.Parser.NALuTypes.html#t:nalu_type/0"},{"type":"type","title":"Membrane.H264.Parser.NALuTypes.vcl_nalu_type/0","doc":"","ref":"Membrane.H264.Parser.NALuTypes.html#t:vcl_nalu_type/0"},{"type":"extras","title":"Membrane H264 Plugin","doc":"# Membrane H264 Plugin\n\n[![Hex.pm](https://img.shields.io/hexpm/v/membrane_h264_plugin.svg)](https://hex.pm/packages/membrane_h264_plugin)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_h264_plugin)\n[![CircleCI](https://circleci.com/gh/membraneframework/membrane_h264_plugin.svg?style=svg)](https://circleci.com/gh/membraneframework/membrane_h264_plugin)\n\nMembrane H264 parser.\nIt is the Membrane element responsible for parsing the incoming H264 stream. The parsing is done as a sequence of the following steps:\n* splitting the H264 stream into stream NAL units, based on the \"Annex B\" of the \"ITU-T Rec. H.264 (01/2012)\" or length prefix defined in \"ISO/IEC 14496-10\"\n* Parsing the NAL unit headers, so that to read the type of the NAL unit\n* Parsing the NAL unit body with the appropriate scheme, based on the NAL unit type read in the step before\n* Aggregating the NAL units into a stream of *access units*\n\nThe output of the element is the incoming binary payload, enriched with the metadata describing the division of the payload into *access units*.\n\nIt is part of [Membrane Multimedia Framework](https://membraneframework.org).","ref":"readme.html"},{"type":"extras","title":"Installation - Membrane H264 Plugin","doc":"The package can be installed by adding `membrane_h264_plugin` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n\t  {:membrane_h264_plugin, \"~> 0.7.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Membrane H264 Plugin","doc":"The following pipeline takes H264 file, parses it, and then decodes it to the raw video.\n\n```elixir\ndefmodule Decoding.Pipeline do\n  use Membrane.Pipeline\n\n  alias Membrane.{File, H264}\n\n  @impl true\n  def handle_init(_ctx, _opts) do\n    structure =\n      child(:source, %File.Source{location: \"test/fixtures/input-10-720p-main.h264\"})\n      |> child(:parser, H264.Parser)\n      |> child(:decoder, H264.FFmpeg.Decoder)\n      |> child(:sink, %File.Sink{location: \"output.raw\"})\n\n    {[spec: structure], nil}\n  end\n\n  @impl true\n  def handle_element_end_of_stream(:sink, _ctx_, state) do\n    {[terminate: :normal], state}\n  end\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Copyright and License - Membrane H264 Plugin","doc":"Copyright 2022, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane_h264_plugin)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane_h264_plugin)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2022 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}